/* Output from p2c, the Pascal-to-C translator */
/* From input file "digiconv.p" */


#include <p2c/p2c.h>


/**********************************************************/
/**                                                      **/
/**             QRSS sound preprocessor                  **/
/**            By Alexander Yurkov RA9MB                 **/
/**                                                      **/
/**********************************************************/


#define pi              3.1415926

#define bufmax          30000




Static short srate;   /*Sampling rate*/
Static double infreqmax;   /*Max Input freq*/
Static double dt;

Static double apodiparam;   /*Apodization parameter*/

Static short srrr;   /*Sampling rate reduction ratio*/
Static double cfreq;   /*Central freq*/

Static double bw;   /*Bandwidth*/
Static double lofreq;   /*Local oscillator freq*/
Static double outfreqmax;   /*Max out freq*/
Static double outfreqmin;   /*Min out freq*/

Static double lo_re, lo_im;   /*Local oscillator voltage*/
Static double lo_mult_re, lo_mult_im;   /*Multiplier for one step of LO*/

Static double filter_re, filter_im;

Static FILE *aufil, *outfil;

Static double inbuf[bufmax + 1];
Static double green_re[bufmax + 1], green_im[bufmax + 1];

Static long snum;
Static unsigned short index_;
Static short minutes;
Static long cycles;
Static double gain;   /*IF gain*/
Static long dispcy;   /*display cycle*/
Static long dispin;   /*display index*/
Static short insigmax, outsigmax;
Static Char aufil_NAME[_FNSIZE];
Static Char outfil_NAME[_FNSIZE];


extern unsigned short SoundStreemReset PP((FILE *sf));

extern unsigned short SoundStreemSetDuplex PP((FILE *sf));

extern unsigned short SoundStreemSync PP((FILE *sf));

extern unsigned short SoundStreemSetRate PP((FILE *sf, int r));

extern unsigned short SoundStreemS16LE PP((FILE *sf));

extern unsigned short SoundStreemS16BE PP((FILE *sf));

extern unsigned short SoundStreemSetChannels PP((FILE *sf, int chan));

extern unsigned short SoundStreemBufInfo PP((FILE *sf));

/* Local variables for GetConfig: */
struct LOC_GetConfig {
  short i, znum;
  double t, time;
} ;

Local Void PulseResponse(LINK)
struct LOC_GetConfig *LINK;
{
  double a;
  short FORLIM;

  a = apodiparam / LINK->znum;
  LINK->time = LINK->znum / bw;
  snum = (long)floor(LINK->time / dt + 0.5) * 2;
  if (snum > bufmax)
    printf("Filter time is too high\n");
  FORLIM = snum;
  for (LINK->i = 0; LINK->i <= FORLIM; LINK->i++) {
    LINK->t = LINK->i * dt - LINK->time;
    green_re[LINK->i] = 2.0 * bw * dt;
    if (LINK->i != snum / 2) {
      green_re[LINK->i] *= sin(pi * LINK->t * bw);
      green_re[LINK->i] /= pi * LINK->t * bw;

      /**** Apodization ****/

      green_re[LINK->i] *= 1.0 / exp(LINK->t * bw * LINK->t * bw * a * a);

    }
    green_im[LINK->i] = -green_re[LINK->i] * sin(2.0 * pi * cfreq * LINK->t);
    green_re[LINK->i] *= cos(2.0 * pi * cfreq * LINK->t);
  }
}




Static Void GetConfig()
{
  struct LOC_GetConfig V;
  FILE *config;   /*config file*/
  Char config_NAME[_FNSIZE];


  config = NULL;
  strcpy(config_NAME, "config.txt");
  if (config != NULL)
    config = freopen(config_NAME, "r", config);
  else
    config = fopen(config_NAME, "r");
  if (config == NULL)
    _EscIO(FileNotFound);

  fscanf(config, "%hd%*[^\n]", &srate);
  getc(config);
  dt = 1.0 / srate;
  infreqmax = srate * 0.5;
  printf("infreqmax: % .5E\n", infreqmax);

  fscanf(config, "%hd%*[^\n]", &srrr);
  getc(config);
  fscanf(config, "%lg%*[^\n]", &cfreq);
  getc(config);
  bw = 0.5 * infreqmax / srrr;
  printf("bandwidth: % .5E\n", bw);
  printf("central freq:% .5E\n", cfreq);
  lofreq = cfreq - bw;
  printf("local oscilator freq: % .5E\n", lofreq);
  outfreqmax = cfreq + 0.5 * bw;
  printf("RXfreqmax: % .5E\n", outfreqmax);
  outfreqmin = cfreq - 0.5 * bw;
  printf("RXfreqmin: % .5E\n", outfreqmin);
  if (lofreq < 0.0) {
    printf("Central frequency is too low\n");
    _Escape(0);
  }
  if (outfreqmax > infreqmax) {
    printf("Central frequency is too high\n");
    _Escape(0);
  }
  lo_mult_re = cos(2.0 * pi * lofreq * dt * srrr);
  lo_mult_im = -sin(2.0 * pi * lofreq * dt * srrr);
  lo_re = 1.0;
  lo_im = 0.0;

  fscanf(config, "%hd%*[^\n]", &V.znum);
  getc(config);
  fscanf(config, "%lg%*[^\n]", &apodiparam);
  getc(config);
  PulseResponse(&V);

  for (V.i = 0; V.i <= bufmax; V.i++)
    inbuf[V.i] = 0.0;
  index_ = 0;
  fscanf(config, "%hd%*[^\n]", &minutes);
  getc(config);
  printf("Minutes to work: %d\n", minutes);
  cycles = minutes * srate * 60 / srrr;
  fscanf(config, "%lg%*[^\n]", &gain);
  getc(config);
  if (config != NULL)
    fclose(config);
  config = NULL;
  dispcy = srate / srrr;
  if (config != NULL)
    fclose(config);
}


main(argc, argv)
int argc;
Char *argv[];
{
  FILE *f;
  long il;
  short i, n, m, j, k;
  unsigned short iw;

  double t_re, t_im, sig;
  short isig;
  unsigned short wsig, buf;
  long FORLIM;
  short FORLIM1;
  Char f_NAME[_FNSIZE];

  PASCAL_MAIN(argc, argv);
  f = NULL;
  outfil = NULL;
  aufil = NULL;
  printf("\n\n");
  GetConfig();
  /**** Debug out: ****/
  /*
               Assign(f,'green.txt');
               ReWrite(f);
               for i:=0 to snum do WriteLn(f,green_re[i],'   ',
                green_im[i],
                Sqrt(
                (green_re[i]*green_re[i])+
                (green_im[i]*green_im[i])
                ));
               Close(f);
   */

  strcpy(outfil_NAME, "digicnvout.raw");
  if (outfil != NULL)
    outfil = freopen(outfil_NAME, "w+b", outfil);
  else
    outfil = fopen(outfil_NAME, "w+b");
  if (outfil == NULL)
    _EscIO(FileNotFound);

  strcpy(aufil_NAME, "/dev/dsp");
  if (aufil != NULL)
    aufil = freopen(aufil_NAME, "r+b", aufil);
  else
    aufil = fopen(aufil_NAME, "r+b");
  if (aufil == NULL)
    _EscIO(FileNotFound);
  if (0 != SoundStreemReset(aufil)) {
    printf("Can not reset sound\n");
    _Escape(0);
  }
  if (0 != SoundStreemSetChannels(aufil, 1)) {
    printf("Can not set one channel\n");
    _Escape(0);
  }
  if (0 != SoundStreemSetRate(aufil, srate)) {
    printf("Can not set rate %d\n", srate);
    _Escape(0);
  }
  if (0 != SoundStreemS16LE(aufil)) {
    printf("Can not set S16LE\n");
    _Escape(0);
  }
  if (0 != SoundStreemSync(aufil)) {
    printf("Can not sync\n");
    _Escape(0);
  }

  /****** Main loop: ********/

  dispin = 0;
  printf("\n\n");
  FORLIM = cycles;

  for (il = 0; il <= FORLIM; il++) {
    FORLIM1 = srrr;
    for (j = 1; j <= FORLIM1; j++) {
      fread(&k, sizeof(unsigned short), 1, aufil);
      inbuf[index_] = k;
      index_ = (index_ + 1) % (snum + 1);
      if (k > insigmax)
	insigmax = k;
      if (k < -insigmax)
	insigmax = -k;

    }

    /**** Get signal ****/


    /**** Filtration ****/

    iw = index_;
    filter_re = 0.0;
    filter_im = 0.0;
    FORLIM1 = snum;
    for (n = 0; n <= FORLIM1; n++) {
      filter_re += green_re[n] * inbuf[iw];
      filter_im += green_im[n] * inbuf[iw];
      iw = (iw + 1) % (snum + 1);
    }


    /**** Local oscillator ****/

    t_re = lo_re * lo_mult_re - lo_im * lo_mult_im;
    t_im = lo_re * lo_mult_im + lo_im * lo_mult_re;
    lo_re = t_re;
    lo_im = t_im;

    /**** Out ****/

    sig = gain * (filter_re * lo_re - filter_im * lo_im);
    if (sig > 30000.0)
      sig = 32760.0;
    if (sig < -30000.0)
      sig = -32760.0;
    isig = (long)floor(sig + 0.5);
    wsig = 0xffffL & isig;
    fwrite(&wsig, sizeof(unsigned short), 1, outfil);
    if (isig > outsigmax)
      outsigmax = isig;
    if (isig < -outsigmax)
      outsigmax = -isig;

    if (dispin < dispcy)
      dispin++;
    else {
      dispin = 0;
      buf = SoundStreemBufInfo(aufil);
      printf("\033Minput:%6d   output:%6d   delaied bytes:%6u\n",
	     insigmax, outsigmax, buf);
      insigmax = 0;
      outsigmax = 0;

      /**** LO amplitude stabilizator ****/

      sig = 1.0 + 0.5 * (lo_re * lo_re + lo_im * lo_im - 1.0);
      lo_re /= sig;
      lo_im /= sig;

    }
  }

  if (outfil != NULL)
    fclose(outfil);
  outfil = NULL;

  if (aufil != NULL)
    fclose(aufil);
  if (outfil != NULL)
    fclose(outfil);
  if (f != NULL)
    fclose(f);
  exit(EXIT_SUCCESS);
}



/* End. */
